function [Sw]=CovCoeffs(Sx,NW,lev,conv)

%CovCoeffs: computes variances and covariances of wavelet coefficients 
%	    
%**********************************************************************
%INPUTS:
% Sx, 		  covariance matrix of dimensions 2^j x 2^j
% NW,             wavelet number (WavBox parameter)
% lev,            number of levels of wavelet transforms (WavBox parameter)
% conv,           convolution version (WavBox parameter)   
%
%OUTPUTS:
% Sw,             variances and covariances of scaling coefficients
%		  at level j-lev and of wavelet coefficients at levels
%		  j-1, ..., j-lev. Sw is in matrix form ordered according
%                 to levels from coarsest (j-lev) to finest (j-1).
%
%USAGE:
% [Sw]=CovCoeffs(Sx,NW,lev,conv);
%
%NOTES:
% This function requires WavBox 4.3b toolbox (copyright Carl Taswell).
% A DWT2 is applied to Sx and a DWT to each row of non-diagonal blocks. 
% When applying the DWT coeffs vectors are reversed so that scaling coeffs 
%  are listed first (this is done to be coherent with the DWT2 - it may have 
%  changed in later upgrades of WavBox).
% Daubechies wavelets with maximum number NW of vanishing moments are used.
% Recommended convolution version is conv='CPF'
%
%REFERENCE:
% Vannucci, M. and Corradi, Fabio (1999).
% Covariance Structure of Wavelet Coefficients: 
% Theory and Models in a Bayesian Perspective. 
% Journal of the Royal Statistical Society, Series B, 61(4), pp.971-986. 
%
%
%Copyright (c) 1997 Marina Vannucci.
%************************************************************************

global MAXLEV

numero=size(Sx,1);
potenza=log2(numero);
Sw=zeros(numero);

%-------------DWT2 of Sx

setwb('OBJTYP','dwt2')
setwb('datsiz',[numero,numero],'deslev',lev,'conver',conv)
setwb('FILCLS','ORTH','FILFAM','DOMP','ANAPAR',NW)
getwb('all');

Sw=dwt(Sx);

%-------------DWT of rows of non-diagonal blocks
 
massimo=MAXLEV;
count=0;
count2=1;
for i=potenza-1:-1:potenza-massimo+1
	tram=zeros(2^i);
	tram=Sw(numero-count+1-2^i:numero-count, ...
	1:numero-count-2^i);        
	setwb('OBJTYP','dwt')
	setwb('datsiz',[2^i,2^i], 'maxlev',(lev-count2), ... 
	'deslev',(lev-count2),'conver','CPF')
	setwb('FILCLS','ORTH','FILFAM','DOMP','ANAPAR',NW)
	%getwb('all');
	fA=tram';
	rovescia=[];
	for j=1:(lev-count2)-1
		[cA,cD] = dndyad(fA);
		fA=cA;
		rovescia=[cD' rovescia']';
	end
	[cA,cD]= dndyad(fA);
	rovescia=[cA',cD',rovescia']';
	Sw(numero-count+1-2^i:numero-count, ...
        1:numero-count-2^i)=rovescia';
	Sw(1:numero-count-2^i,numero-count+1-2^i:numero-count)=...
	rovescia;
	count=count+2^i;
	count2=count2+1;
end

