function [Uni,UniD,GammaD,logProb,logProbD,PostGamD,AD,S]= ...
bvsme_wav(gamprec,X,Y,delta,k,w,Hg)

%bvsme_wav: Bayesian Variable Selection with selection prior with 
% 	    non-diagonal cov. matrix Hg and Metropolis search.
%			     Main Program 
%***********************************************************************
%INPUTS:
% gamprec,      starting binary vector.
%               If gamprec=[] the program asks for r to select a starting
%               vector with first r elements equal to 1 
% X,            independent variables
%		(matrix of wavelet coefficients in the reference given below)
% Y,            response variables, matrix (n by q)
% delta, k,     hyperparameters of Inverse Wishart prior
% w,		hyperparameter of prior on latent binary vector
%		- vector of parameters w_j, j=1...p (Bernoulli priors)
%		- scalar in the case of equal parameters w_j=w 
%		  (Binomial prior with mean pw)
%		- vector of two values for a Beta-Binomial prior
% 		(with mean=pw(1), w(2)=sum of the two parameters of the Beta)
% Hg,   	hyperparameter of the normal prior on regression coefficients
%		- full var-cov matrix
%		(computed via CovCoeffs.m in the reference given below)
%
%OUTPUTS:
% Uni,		vector with number of ones of visited models
% UniD,		vector with number of ones of distinct visited models
% GammaD,	distinct visited models, ordered according to
%		their (normalized) relative posterior probability
%		(cell array of strings)
% logProb,      log-relative posterior probabilities 
%		of all visited models
% logProbD      log-relative posterior probabilities
%               of distinct visited models
% PostGamD	normalized ordered relative probabilities 
%		of distinct visited models 
% AD, S,	iteration numbers of accepted additions/deletions (AD) 
%		and swaps (S)
%USAGE:
% [Uni,UniD,GammD,logProb,logProbD,PostGamD,AD,S]= ...
%	bvsme_wav(gamprec,X,Y,delta,k,w,Hg)
%
%NOTES:
% Data must be centered.
% The programs asks for the Metropolis parameters
%  (initial number of variables included, probabilities of  
%  deletion/addition and swap moves, total number of iterations).
% QR decomposition is performed at every iteration.
%  The code is designed for a non-diagonal prior (Hg full var-cov matrix).
%  This prior does not allow fast QRdelete/insert updating.
%  Code is therefore sub-optimal if used with diagonal  Hg
% GammaD is in cell array format 
%  (elements are strings --> required by unique.m)
% Search for replicates among visited accepted models of same size
% Functions called: gofg_wav, metro_wav, replichef, probordf
%
%REFERENCE: 
% Brown, P.J., Fearn, T. and Vannucci, M. (2001).
% Bayesian Wavelet Regression on Curves with Applications 
% to a Spectroscopic Calibration Problem. 
% Journal of the American Statistical Society, 96, pp.
%
%
%Copyright (c) 1998-2000 Marina Vannucci.
%**************************************************************************

[nOss p]=size(X);

[q]=size(Y,2); 
        % p independent variables, q responses, nOss observations

disp(' ')
disp('------- METROPOLIS --------')
disp(' ')


if isempty(gamprec)
	disp(' ')
	disp('-- Starting binary vector with first r elements equal to 1');
	r=input('   please provide r, r=?');	
	gamprec=[ones(1,r) zeros(1,p-r)];
else
end

disp(' ')
disp('-- The Metropolis search moves by')
disp('   A/D)adding or deleting a variable (with probability P)')
disp('                            or') 
disp('   S)swapping two variables (with probability 1-P)')
pPar=input('   please provide P, probabitity of moves A/D, P=?');
disp(' ')
nmetr=input('-- Number ot total iterations (multiple than 500)=?');	
disp(' ')
disp('------- RUNNING METROPOLIS ...')
disp(' ')
nPar=500;
GammaD{1}=num2str(find(gamprec));
Uni(1)=r;
UniD(1)=r;
logProb=zeros(1,nmetr+1);
AD=[];
S=[];
conta=1;
contaD=1;

%-------------------- initial gamma
	
	[probprec] = gofg_wav(gamprec,X,Y,k,delta,w,Hg);
	logProb(conta)= probprec;
	logProbD(contaD)=probprec;

%-------------------- Metropolis (nPar iterations)

while conta<nmetr+1

       	[UniSt,ProbSt, GammaSt, storeAD, storeS]= metro_wav(....
	gamprec,probprec,X,Y,k,delta,w,Hg,nPar,pPar);
	logProb(conta+1:conta+nPar)=ProbSt;
	Uni=[Uni UniSt];
	indiciD=sort([storeAD storeS]);
	GammaD=cat(1,GammaD, GammaSt(indiciD));
	logProbD=[logProbD, ProbSt(indiciD)];	
	UniD=[UniD, UniSt(indiciD)];
	AD=cat(2, AD, (conta-1)+storeAD);
	S=cat(2, S, (conta-1)+storeS);
	conta=conta+nPar;
	contaD=length(logProbD);
	gamprec=zeros(1,p);
	gamprec(str2num(GammaSt{nPar}))=ones(1,length(str2num(GammaSt{nPar})));
	disp(' ')
       	disp('Number of iterations:')
       	disp(conta-1)
       	disp('...')
end
disp(' ')
disp('------- Searching for replicates ...')
disp(' ')

%------------------- Search for replications
[UniD,GammaD, logProbD]=replichef(UniD,GammaD, logProbD);

%------------------- Normalized posterior and  marginal probs.
%                    Ordering of distinct visited vectors according
%                    to probability

[UniD,GammaD, logProbD, PostGamD]=probordf(UniD,logProbD, GammaD);


disp(' ')
disp('--- End.') 
