function [r] = gofg_wav(gamma,X,Y,k,delta,w,Hg)

%gofg_wav: log-relative posterior probability of gamma. 
%	    Selection prior is with fast form of updating
%*******************************************************************
%INPUTS:
% gamma,          binary vector
% X, 		  independent variables, n by p
% Y,              augmented response matrix, (n+p) by q
% k,delta,        hyperparameters of Inverse Wishart prior
% w,              hyperparameters of prior on latent binary vector
% Hg,             var-cov matrix of normal prior on regression coefficients 
%
%OUTPUTS:
% r,              log-relative posterior probability of gamma
%
%USAGE:
% [r]= gofg_wav(gamma,X,Y,delta,k,w,Hg);
%
%NOTES:
% Called by bvsme_wav and metro_wav
% QR decomposition is performed.
% Designed for a non-diagonal prior (Hg full var-cov matrix).
% This prior does not allow fast QRdelete/insert updating.
%
%REFERENCE:
% Brown, P.J., Fearn, T. and Vannucci, M. (2001).
% Bayesian Wavelet Regression on Curves with Applications 
% to a Spectroscopic Calibration Problem. 
% Journal of the American Statistical Society, 96, pp.
%
%
%Copyright (c) 1998-2000 Marina Vannucci.
%**********************************************************************

[nOss p]=size(X);
[q]=size(Y,2);
Qk=k*eye(q);                         
I=find(gamma);
nonnulle=length(I);
XGam=X(:,I);	 
Y=cat(1,Y,zeros(nonnulle,q));
sotto=eye(nonnulle);
moltiplica=sqrtm(Hg(I,I));;		
Xtilde=cat(1,XGam*moltiplica, sotto);
[Qqr Rqr]=qr(Xtilde); 			
		             
		
%---------------------------------- prior on gamma
if (length(w)==1 | length(w)==p)
	prog=sum(log((w.^gamma).*(1-w).^(1-gamma)));
elseif length(w)==2
	alpha=w(2)*w(1);	% Beta prior (mean=pw(1), w(2)=alpha+beta) 
	betapar=w(2)*(1-w(1));
	gami=sum(gamma);
	prog=(gammaln(alpha+gami)+gammaln(p+betapar-gami))- ...
	      gammaln(p+betapar+alpha);
end

%----------------------------------- log-relative posterior prob of gamma
sub=Qqr(:,nonnulle+1:nOss+nonnulle)'*Y;	% Seber, page 498
RSP=sub'*sub;                       	% residual sum of products
Qstar=Qk+RSP;                   % q by q
switch nonnulle
	case 1
        	detH0K=Rqr(1,1)^2;
	otherwise
        	detH0K=prod(diag(Rqr).^2);
end			
r= -(q/2)*log(detH0K)-((delta+nOss+q-1)/2)*log(det(Qstar))+prog;

