function [BayesPred,LSPred,ILS,IB]=pred_wav(X,Y,Xf,Yf,PostGamD,GammaD,numero,Hg)

%pred_wav: Bayesian Variable Selection with non-diagonal selection prior
%         Prediction by Least Squares and Bayesian Model Averaging 
%***********************************************************************
%INPUTS:
% X,Y,		training data
% Xf,Yf,	validation data
% PostGamD,	vector of posterior probability of visited models
% GammaD, 	matrix or cell of distinct ordered visited models
% numero,	number of best models for model avaraging
% Hg,	        var-cov matrix
%
%OUTPUTS:
% BayesPred	Bayes prediction with the 'numero' most likely models
% LSPred        Least Squares Prediction with the best model
% ILS           indices of selected variables for LS prediction
% IB		indices of selected variables for Bayes prediction
%
%USAGE:
% [BayesPred,LSPred,ILS,IB]=predico(X,Y,Xf,Yf,PostGamD,GammaD,numero,Hg);
%
%NOTES:
% Training Data must be centered
% Validation data must be centered with training means
%
%REFERENCE:
% Brown, P.J., Fearn, T. and Vannucci, M. (2001).
% Bayesian Wavelet Regression on Curves with Applications 
% to a Spectroscopic Calibration Problem. 
% Journal of the American Statistical Society, 96, pp.
%
%
%Copyright (c) 1998-2000 Marina Vannucci.
%**********************************************************************

switch class(GammaD)
	case 'cell'
		ILS=str2num(GammaD{1});			
	otherwise
		ILS=find(GammaD(1,:));
end		
[nOssP,p]=size(Xf);
[nOss]=size(X,1);
[q]=size(Y,2);

%-------------------------------------- LS prediction with best model
		
nonnulle=length(ILS);
Xfg=Xf(:,ILS);
Xg=X(:,ILS);
if nonnulle>nOss
	disp(' ')
	disp('Least square prediction with more variables')
	disp('than observations. Use generalized inverse')
	disp(' ')
	Beta=pinv(Xg'*Xg)*Xg'*Y;
	Yfpred=Xfg*Beta;
	residual=Yf-Yfpred;
else 
	[Qqr Rqr]=qr(Xg);
	Zeta=Qqr(:,1:nonnulle)'*Y;
	U=Qqr(:,1:nonnulle)'*Xg;
	Beta=inv(U)*Zeta;   
	Yfpred=Xfg*Beta;
	residual=Yf-Yfpred;
end		
LSPred=(diag(residual'*residual)./nOssP)';

%-------------------------------------- BMA prediction with 'numero' models
Ytilde=cat(1,Y, zeros(p,q));				
Yfpred=zeros(nOssP,q);
costante=sum(PostGamD(1:numero));
IB=[];
for i=1:numero
	switch class(GammaD)
		case 'cell'
			Ii=str2num(GammaD{i});
		otherwise
			Ii=find(GammaD(i,:));
	end
	nonnulle=length(Ii);
			%--------------------------- Hg full matrix 
	moltiplica=sqrtm(Hg(Ii,Ii));
	XH0=X(:,Ii)*moltiplica;
	sotto=zeros(p,nonnulle);
	sotto(Ii,:)=eye(nonnulle);    
	Xtilde=cat(1, XH0, sotto);
	Xfg=Xf(:,Ii);
	Beta=moltiplica*inv(Xtilde'*Xtilde)*Xtilde'*Ytilde;
	Yfpred=Yfpred+Xfg*Beta*PostGamD(i)/costante;
	IB=union(IB,Ii);
end
residual=Yf-Yfpred;
BayesPred=(diag(residual'*residual)./nOssP)';
